@@ -71,67 +71,6 @@ static dvdnav_status_t dvdnav_clear(dvdnav_t * this) {
   return DVDNAV_STATUS_OK;
 }
 
-dvdnav_status_t dvdnav_dup(dvdnav_t **dest, dvdnav_t *src) {
-  dvdnav_t *this;
-
-  (*dest) = NULL;
-  this = (dvdnav_t*)malloc(sizeof(dvdnav_t));
-  if(!this)
-    return DVDNAV_STATUS_ERR;
-
-  memcpy(this, src, sizeof(dvdnav_t));
-  this->file = NULL;
-
-  pthread_mutex_init(&this->vm_lock, NULL);
-
-  this->vm = vm_new_copy(src->vm);
-  if(!this->vm) {
-    printerr("Error initialising the DVD VM.");
-    pthread_mutex_destroy(&this->vm_lock);
-    free(this);
-    return DVDNAV_STATUS_ERR;
-  }
-
-  /* Start the read-ahead cache. */
-  this->cache = dvdnav_read_cache_new(this);
-
-  (*dest) = this;
-  return DVDNAV_STATUS_OK;
-}
-
-dvdnav_status_t dvdnav_free_dup(dvdnav_t *this) {
-
-#ifdef LOG_DEBUG
-  fprintf(MSG_OUT, "libdvdnav: free_dup:called\n");
-#endif
-
-  if (this->file) {
-    pthread_mutex_lock(&this->vm_lock);
-    DVDCloseFile(this->file);
-#ifdef LOG_DEBUG
-    fprintf(MSG_OUT, "libdvdnav: close:file closing\n");
-#endif
-    this->file = NULL;
-    pthread_mutex_unlock(&this->vm_lock);
-  }
-
-  /* Free the VM */
-  if(this->vm)
-    vm_free_copy(this->vm);
-
-  pthread_mutex_destroy(&this->vm_lock);
-
-  /* We leave the final freeing of the entire structure to the cache,
-   * because we don't know, if there are still buffers out in the wild,
-   * that must return first. */
-  if(this->cache)
-    dvdnav_read_cache_free(this->cache);
-  else
-    free(this);
-
-  return DVDNAV_STATUS_OK;
-}
-
 dvdnav_status_t dvdnav_open(dvdnav_t** dest, const char *path) {
   dvdnav_t *this;
   struct timeval time;
@@ -264,12 +203,12 @@ int64_t dvdnav_convert_time(dvd_time_t *time) {
   int64_t result;
   int64_t frames;
 
-  result  = ((int64_t)(time->hour    >> 4  )) * 10 * 60 * 60 * 90000;
-  result += ((int64_t)(time->hour    & 0x0f))      * 60 * 60 * 90000;
-  result += ((int64_t)(time->minute  >> 4  ))      * 10 * 60 * 90000;
-  result += ((int64_t)(time->minute  & 0x0f))           * 60 * 90000;
-  result += ((int64_t)(time->second  >> 4  ))           * 10 * 90000;
-  result += ((int64_t)(time->second  & 0x0f))                * 90000;
+  result  = (time->hour    >> 4  ) * 10 * 60 * 60 * 90000ull;
+  result += (time->hour    & 0x0f)      * 60 * 60 * 90000;
+  result += (time->minute  >> 4  )      * 10 * 60 * 90000;
+  result += (time->minute  & 0x0f)           * 60 * 90000;
+  result += (time->second  >> 4  )           * 10 * 90000;
+  result += (time->second  & 0x0f)                * 90000;
 
   frames  = ((time->frame_u & 0x30) >> 4) * 10;
   frames += ((time->frame_u & 0x0f)     )     ;
@@ -580,10 +519,8 @@ dvdnav_status_t dvdnav_get_next_cache_block(dvdnav_t *this, uint8_t **buf,
     return DVDNAV_STATUS_OK;
   }
 
-  /* Check to see if we need to change the currently opened VOB or open
-   * a new one because we don't currently have an opened VOB. */
-  if((this->file == NULL) ||
-     (this->position_current.vts != this->position_next.vts) ||
+  /* Check to see if we need to change the currently opened VOB */
+  if((this->position_current.vts != this->position_next.vts) ||
      (this->position_current.domain != this->position_next.domain)) {
     dvd_read_domain_t domain;
     int32_t vtsN;
@@ -599,16 +536,16 @@ dvdnav_status_t dvdnav_get_next_cache_block(dvdnav_t *this, uint8_t **buf,
 
     /* Use the DOMAIN to find whether to open menu or title VOBs */
     switch(this->position_next.domain) {
-    case DVD_DOMAIN_FirstPlay:
-    case DVD_DOMAIN_VMGM:
+    case FP_DOMAIN:
+    case VMGM_DOMAIN:
       domain = DVD_READ_MENU_VOBS;
       vtsN = 0;
       break;
-    case DVD_DOMAIN_VTSMenu:
+    case VTSM_DOMAIN:
       domain = DVD_READ_MENU_VOBS;
       vtsN = this->position_next.vts;
       break;
-    case DVD_DOMAIN_VTSTitle:
+    case VTS_DOMAIN:
       domain = DVD_READ_TITLE_VOBS;
       vtsN = this->position_next.vts;
       break;
@@ -719,7 +656,7 @@ dvdnav_status_t dvdnav_get_next_cache_block(dvdnav_t *this, uint8_t **buf,
     fprintf(MSG_OUT, "libdvdnav: SPU_CLUT_CHANGE\n");
 #endif
     (*len) = 16 * sizeof(uint32_t);
-    memcpy(*buf, state->pgc->palette, sizeof(state->pgc->palette));
+    memcpy(*buf, &(state->pgc->palette), 16 * sizeof(uint32_t));
     this->spu_clut_changed = 0;
     pthread_mutex_unlock(&this->vm_lock);
     return DVDNAV_STATUS_OK;
@@ -795,15 +732,16 @@ dvdnav_status_t dvdnav_get_next_cache_block(dvdnav_t *this, uint8_t **buf,
       /* we are about to leave a cell, so a lot of state changes could occur;
        * under certain conditions, the application should get in sync with us before this,
        * otherwise it might show stills or menus too shortly */
-      if ((this->position_current.still || this->pci.hli.hl_gi.hli_ss) && !this->sync_wait_skip)
+      if ((this->position_current.still || this->pci.hli.hl_gi.hli_ss) && !this->sync_wait_skip) {
         this->sync_wait = 1;
-
-      if(!this->position_current.still || this->skip_still ) {
-	/* no active cell still -> get us to the next cell */
-	vm_get_next_cell(this->vm);
-	this->position_current.still = 0; /* still gets activated at end of cell */
-	this->skip_still = 0;
-	this->sync_wait_skip = 0;
+      } else {
+	if( this->position_current.still == 0 || this->skip_still ) {
+	  /* no active cell still -> get us to the next cell */
+	  vm_get_next_cell(this->vm);
+	  this->position_current.still = 0; /* still gets activated at end of cell */
+	  this->skip_still = 0;
+	  this->sync_wait_skip = 0;
+	}
       }
       /* handle related state changes in next iteration */
       (*event) = DVDNAV_NOP;
@@ -1141,7 +1079,7 @@ int8_t dvdnav_get_active_spu_stream(dvdnav_t *this) {
   return retval;
 }
 
-static int8_t dvdnav_is_domain(dvdnav_t *this, DVDDomain_t domain) {
+static int8_t dvdnav_is_domain(dvdnav_t *this, domain_t domain) {
   int8_t        retval;
 
   if(!this->started) {
@@ -1158,19 +1096,19 @@ static int8_t dvdnav_is_domain(dvdnav_t *this, DVDDomain_t domain) {
 
 /* First Play domain. (Menu) */
 int8_t dvdnav_is_domain_fp(dvdnav_t *this) {
-  return dvdnav_is_domain(this, DVD_DOMAIN_FirstPlay);
+  return dvdnav_is_domain(this, FP_DOMAIN);
 }
 /* Video management Menu domain. (Menu) */
 int8_t dvdnav_is_domain_vmgm(dvdnav_t *this) {
-  return dvdnav_is_domain(this, DVD_DOMAIN_VMGM);
+  return dvdnav_is_domain(this, VMGM_DOMAIN);
 }
 /* Video Title Menu domain (Menu) */
 int8_t dvdnav_is_domain_vtsm(dvdnav_t *this) {
-  return dvdnav_is_domain(this, DVD_DOMAIN_VTSMenu);
+  return dvdnav_is_domain(this, VTSM_DOMAIN);
 }
 /* Video Title domain (playing movie). */
 int8_t dvdnav_is_domain_vts(dvdnav_t *this) {
-  return dvdnav_is_domain(this, DVD_DOMAIN_VTSTitle);
+  return dvdnav_is_domain(this, VTS_DOMAIN);
 }
 
 /* Generally delegate angle information handling to VM */
