@@ -89,9 +89,6 @@ typedef int32_t dvdnav_status_t;
  */
 dvdnav_status_t dvdnav_open(dvdnav_t **dest, const char *path);
 
-dvdnav_status_t dvdnav_dup(dvdnav_t **dest, dvdnav_t *src);
-dvdnav_status_t dvdnav_free_dup(dvdnav_t *this);
-
 /*
  * Closes a dvdnav_t previously opened with dvdnav_open(), freeing any
  * memory associated with it.
@@ -373,7 +370,7 @@ dvdnav_status_t dvdnav_part_search(dvdnav_t *self, int32_t part);
  * fcntl.h.
  */
 dvdnav_status_t dvdnav_sector_search(dvdnav_t *self,
-				     int64_t offset, int32_t origin);
+				     uint64_t offset, int32_t origin);
 
 /*
  returns the current stream time in PTS ticks as reported by the IFO structures
@@ -385,8 +382,7 @@ int64_t dvdnav_get_current_time(dvdnav_t *self);
  * Stop playing the current position and start playback of the title
  * from the specified timecode.
  *
- * Currently implemented using interpolation. That interpolation is slightly
- * inaccurate.
+ * Currently implemented using interpolation, which is slightly inaccurate.
  */
 dvdnav_status_t dvdnav_time_search(dvdnav_t *self,
 				   uint64_t time);
