@@ -73,7 +73,7 @@ static dvdnav_status_t dvdnav_scan_admap(dvdnav_t *this, int32_t domain, uint32_
   }
   if(admap) {
     uint32_t address = 0;
-    uint32_t vobu_start, next_vobu = 0;
+    uint32_t vobu_start, next_vobu;
     int admap_entries = (admap->last_byte + 1 - VOBU_ADMAP_SIZE)/VOBU_ADMAP_SIZE;
 
     /* Search through ADMAP for best sector */
@@ -403,7 +403,8 @@ dvdnav_status_t dvdnav_next_pg_search(dvdnav_t *this) {
   pthread_mutex_lock(&this->vm_lock);
   if(!this->vm->state.pgc) {
     printerr("No current PGC.");
-    goto fail;
+    pthread_mutex_unlock(&this->vm_lock);
+    return DVDNAV_STATUS_ERR;
   }
 
 #ifdef LOG_DEBUG
@@ -411,25 +412,17 @@ dvdnav_status_t dvdnav_next_pg_search(dvdnav_t *this) {
 #endif
   /* make a copy of current VM and try to navigate the copy to the next PG */
   try_vm = vm_new_copy(this->vm);
-  if (try_vm == NULL) {
-    printerr("Unable to copy the VM.");
-    goto fail;
-  }
-
   if (!vm_jump_next_pg(try_vm) || try_vm->stopped) {
     vm_free_copy(try_vm);
     /* next_pg failed, try to jump at least to the next cell */
     try_vm = vm_new_copy(this->vm);
-    if (try_vm == NULL) {
-      printerr("Unable to copy the VM.");
-      goto fail;
-    }
     vm_get_next_cell(try_vm);
     if (try_vm->stopped) {
       vm_free_copy(try_vm);
       fprintf(MSG_OUT, "libdvdnav: next chapter failed.\n");
       printerr("Skip to next chapter failed.");
-      goto fail;
+      pthread_mutex_unlock(&this->vm_lock);
+      return DVDNAV_STATUS_ERR;
     }
   }
   this->cur_cell_time = 0;
@@ -444,10 +437,6 @@ dvdnav_status_t dvdnav_next_pg_search(dvdnav_t *this) {
   pthread_mutex_unlock(&this->vm_lock);
 
   return DVDNAV_STATUS_OK;
-
-fail:
-  pthread_mutex_unlock(&this->vm_lock);
-  return DVDNAV_STATUS_ERR;
 }
 
 dvdnav_status_t dvdnav_menu_call(dvdnav_t *this, DVDMenuID_t menu) {
@@ -456,17 +445,13 @@ dvdnav_status_t dvdnav_menu_call(dvdnav_t *this, DVDMenuID_t menu) {
   pthread_mutex_lock(&this->vm_lock);
   if(!this->vm->state.pgc) {
     printerr("No current PGC.");
-    goto fail;
+    pthread_mutex_unlock(&this->vm_lock);
+    return DVDNAV_STATUS_ERR;
   }
 
   this->cur_cell_time = 0;
   /* make a copy of current VM and try to navigate the copy to the menu */
   try_vm = vm_new_copy(this->vm);
-  if (try_vm == NULL) {
-    printerr("Unable to copy VM.");
-    goto fail;
-  }
-
   if ( (menu == DVD_MENU_Escape) && (this->vm->state.domain != VTS_DOMAIN)) {
     /* Try resume */
     if (vm_jump_resume(try_vm) && !try_vm->stopped) {
@@ -492,12 +477,9 @@ dvdnav_status_t dvdnav_menu_call(dvdnav_t *this, DVDMenuID_t menu) {
   } else {
     vm_free_copy(try_vm);
     printerr("No such menu or menu not reachable.");
-    goto fail;
+    pthread_mutex_unlock(&this->vm_lock);
+    return DVDNAV_STATUS_ERR;
   }
-
-fail:
-  pthread_mutex_unlock(&this->vm_lock);
-  return DVDNAV_STATUS_ERR;
 }
 
 dvdnav_status_t dvdnav_get_position(dvdnav_t *this, uint32_t *pos,
@@ -634,16 +616,40 @@ uint32_t dvdnav_describe_title_chapters(dvdnav_t *this, int32_t title, uint64_t
   length = 0;
   for(i=0; i<parts; i++) {
     uint32_t cellnr, endcellnr;
+    if (ptt[i].pgcn == 0 || ptt[i].pgcn > ifo->vts_pgcit->nr_of_pgci_srp) {
+      printerr("PGCN out of bounds.");
+      continue;
+    }
+    if (ifo->vts_pgcit->pgci_srp[ptt[i].pgcn-1].pgc_start_byte >= ifo->vts_pgcit->last_byte) {
+      printerr("PGC start out of bounds");
+      continue;
+    }
     pgc = ifo->vts_pgcit->pgci_srp[ptt[i].pgcn-1].pgc;
-    if(ptt[i].pgn > pgc->nr_of_programs) {
+    if (pgc == NULL) {
+      printerr("PGC missing.");
+      continue;
+    }
+    if (pgc->program_map == NULL) {
+      printerr("Program map missing.");
+      continue;
+    }
+    if(ptt[i].pgn == 0 || ptt[i].pgn > pgc->nr_of_programs) {
       printerr("WRONG part number.");
       goto fail;
     }
 
+    if (pgc->nr_of_cells == 0) {
+      printerr("Number of cells cannot be 0");
+      continue;
+    }
     if ((cellnr = pgc->program_map[ptt[i].pgn-1]) == 0) {
       printerr("Cell new row cannot be 0");
       continue;
     }
+    if (pgc->cell_playback == NULL) {
+      printerr("Cell missing");
+      continue;
+    }
 
     if(ptt[i].pgn < pgc->nr_of_programs)
       endcellnr = pgc->program_map[ptt[i].pgn];
