@@ -101,8 +101,8 @@ static int get_PGCN(vm_t *vm);
 static pgcit_t* get_MENU_PGCIT(vm_t *vm, ifo_handle_t *h, uint16_t lang);
 static pgcit_t* get_PGCIT(vm_t *vm);
 
+
 /* Helper functions */
-static void vm_close(vm_t *vm);
 
 #ifdef TRACE
 static void vm_print_current_domain_state(vm_t *vm) {
@@ -162,86 +162,63 @@ static int os2_open(const char *name, int oflag)
 }
 #endif
 
-static int dvd_read_name(char *name, char *serial, const char *device) {
-  /* Because we are compiling with _FILE_OFFSET_BITS=64
-   * all off_t are 64bit.
-   */
-  off_t off;
-  ssize_t read_size = 0;
-  int fd = -1, i;
-  uint8_t data[DVD_VIDEO_LB_LEN];
+static void dvd_read_name(char *name, char *serial, const char *device) {
+    /* Because we are compiling with _FILE_OFFSET_BITS=64
+     * all off_t are 64bit.
+     */
+    off_t off;
+    int fd, i;
+    uint8_t data[DVD_VIDEO_LB_LEN];
 
-  /* Read DVD name */
-  if (device == NULL) {
-    fprintf(MSG_OUT, "libdvdnav: Device name string NULL\n");
-    goto fail;
-  }
-  if ((fd = open(device, O_RDONLY)) == -1) {
-    fprintf(MSG_OUT, "libdvdnav: Unable to open device file %s.\n", device);
-    goto fail;
-  }
-
-  if ((off = lseek( fd, 32 * (off_t) DVD_VIDEO_LB_LEN, SEEK_SET )) == (off_t) - 1) {
-    fprintf(MSG_OUT, "libdvdnav: Unable to seek to the title block %u.\n", 32);
-    goto fail;
-  }
-
-  if( off != ( 32 * (off_t) DVD_VIDEO_LB_LEN ) ) {
-    fprintf(MSG_OUT, "libdvdnav: Can't seek to block %u\n", 32 );
-    goto fail;
-  }
-
-  if ((read_size = read( fd, data, DVD_VIDEO_LB_LEN )) == -1) {
-    fprintf(MSG_OUT, "libdvdnav: Can't read name block. Probably not a DVD-ROM device.\n");
-    goto fail;
-  }
-
-  close(fd);
-  fd = -1;
-  if (read_size != DVD_VIDEO_LB_LEN) {
-    fprintf(MSG_OUT, "libdvdnav: Can't read name block. Probably not a DVD-ROM device.\n");
-    goto fail;
-  }
-
-  fprintf(MSG_OUT, "libdvdnav: DVD Title: ");
-  for(i=25; i < 73; i++ ) {
-    if((data[i] == 0)) break;
-    if((data[i] > 32) && (data[i] < 127)) {
-      fprintf(MSG_OUT, "%c", data[i]);
+    /* Read DVD name */
+    fd = open(device, O_RDONLY);
+    if (fd > 0) {
+      off = lseek( fd, 32 * (off_t) DVD_VIDEO_LB_LEN, SEEK_SET );
+      if( off == ( 32 * (off_t) DVD_VIDEO_LB_LEN ) ) {
+        off = read( fd, data, DVD_VIDEO_LB_LEN );
+        if (off == ( (off_t) DVD_VIDEO_LB_LEN )) {
+          fprintf(MSG_OUT, "libdvdnav: DVD Title: ");
+          for(i=25; i < 73; i++ ) {
+            if((data[i] == 0)) break;
+            if((data[i] > 32) && (data[i] < 127)) {
+              fprintf(MSG_OUT, "%c", data[i]);
+            } else {
+              fprintf(MSG_OUT, " ");
+            }
+          }
+          strncpy(name, (char*) &data[25], 48);
+          name[48] = 0;
+          fprintf(MSG_OUT, "\nlibdvdnav: DVD Serial Number: ");
+          for(i=73; i < 89; i++ ) {
+            if((data[i] == 0)) break;
+            if((data[i] > 32) && (data[i] < 127)) {
+              fprintf(MSG_OUT, "%c", data[i]);
+            } else {
+              fprintf(MSG_OUT, " ");
+            }
+          }
+          strncpy(serial, (char*) &data[73], (i-73));
+          serial[14] = 0;
+          fprintf(MSG_OUT, "\nlibdvdnav: DVD Title (Alternative): ");
+          for(i=89; i < 128; i++ ) {
+            if((data[i] == 0)) break;
+            if((data[i] > 32) && (data[i] < 127)) {
+              fprintf(MSG_OUT, "%c", data[i]);
+            } else {
+              fprintf(MSG_OUT, " ");
+            }
+          }
+          fprintf(MSG_OUT, "\n");
+        } else {
+          fprintf(MSG_OUT, "libdvdnav: Can't read name block. Probably not a DVD-ROM device.\n");
+        }
+      } else {
+        fprintf(MSG_OUT, "libdvdnav: Can't seek to block %u\n", 32 );
+      }
+      close(fd);
     } else {
-      fprintf(MSG_OUT, " ");
-    }
+    fprintf(MSG_OUT, "NAME OPEN FAILED\n");
   }
-  strncpy(name, (char*) &data[25], 48);
-  name[48] = 0;
-  fprintf(MSG_OUT, "\nlibdvdnav: DVD Serial Number: ");
-  for(i=73; i < 89; i++ ) {
-    if((data[i] == 0)) break;
-    if((data[i] > 32) && (data[i] < 127)) {
-      fprintf(MSG_OUT, "%c", data[i]);
-    } else {
-      fprintf(MSG_OUT, " ");
-    }
-  }
-  strncpy(serial, (char*) &data[73], (i-73));
-  serial[14] = 0;
-  fprintf(MSG_OUT, "\nlibdvdnav: DVD Title (Alternative): ");
-  for(i=89; i < 128; i++ ) {
-    if((data[i] == 0)) break;
-    if((data[i] > 32) && (data[i] < 127)) {
-      fprintf(MSG_OUT, "%c", data[i]);
-    } else {
-      fprintf(MSG_OUT, " ");
-    }
-  }
-  fprintf(MSG_OUT, "\n");
-  return 1;
-
-fail:
-  if (fd >= 0)
-    close(fd);
-
-  return 0;
 }
 
 static int ifoOpenNewVTSI(vm_t *vm, dvd_reader_t *dvd, int vtsN) {
@@ -286,11 +263,11 @@ static int ifoOpenNewVTSI(vm_t *vm, dvd_reader_t *dvd, int vtsN) {
 /* Initialisation & Destruction */
 
 vm_t* vm_new_vm() {
-  return (vm_t*)calloc(sizeof(vm_t), sizeof(char));
+  return (vm_t*)calloc(1, sizeof(vm_t));
 }
 
 void vm_free_vm(vm_t *vm) {
-  vm_close(vm);
+  vm_stop(vm);
   free(vm);
 }
 
@@ -316,12 +293,6 @@ dvd_reader_t *vm_get_dvd_reader(vm_t *vm) {
 /* Basic Handling */
 
 int vm_start(vm_t *vm) {
-  if (vm->stopped) {
-    if (!vm_reset(vm, NULL))
-      return 0;
-
-    vm->stopped = 0;
-  }
   /* Set pgc to FP (First Play) pgc */
   set_FP_PGC(vm);
   process_command(vm, play_PGC(vm));
@@ -329,10 +300,6 @@ int vm_start(vm_t *vm) {
 }
 
 void vm_stop(vm_t *vm) {
-  vm->stopped = 1;
-}
-
-static void vm_close(vm_t *vm) {
   if(vm->vmgi) {
     ifoClose(vm->vmgi);
     vm->vmgi=NULL;
@@ -382,9 +349,11 @@ int vm_reset(vm_t *vm, const char *dvdroot) {
 
   (vm->state).vtsN               = -1;
 
+  vm->hop_channel                = 0;
+
   if (vm->dvd && dvdroot) {
     /* a new dvd device has been requested */
-    vm_close(vm);
+    vm_stop(vm);
   }
   if (!vm->dvd) {
     vm->dvd = DVDOpen(dvdroot);
@@ -422,8 +391,8 @@ int vm_reset(vm_t *vm, const char *dvdroot) {
       /* return 0; Not really used for now.. */
     }
     /* ifoRead_TXTDT_MGI(vmgi); Not implemented yet */
-    if (dvd_read_name(vm->dvd_name, vm->dvd_serial, dvdroot) == 1)
-      vm->map  = remap_loadmap(vm->dvd_name);
+    dvd_read_name(vm->dvd_name, vm->dvd_serial, dvdroot);
+    vm->map  = remap_loadmap(vm->dvd_name);
   }
   if (vm->vmgi) {
     int i, mask;
@@ -446,8 +415,7 @@ vm_t *vm_new_copy(vm_t *source) {
   int pgcN = get_PGCN(source);
   int pgN  = (source->state).pgN;
 
-  if (target == NULL || pgcN == 0)
-    goto fail;
+  assert(pgcN);
 
   memcpy(target, source, sizeof(vm_t));
 
@@ -457,22 +425,15 @@ vm_t *vm_new_copy(vm_t *source) {
   if (vtsN > 0) {
     (target->state).vtsN = 0;
     if (!ifoOpenNewVTSI(target, target->dvd, vtsN))
-      goto fail;
+      assert(0);
 
     /* restore pgc pointer into the new vtsi */
     if (!set_PGCN(target, pgcN))
-      goto fail;
-
+      assert(0);
     (target->state).pgN = pgN;
   }
 
   return target;
-
-fail:
-  if (target != NULL)
-    vm_free_vm(target);
-
-  return NULL;
 }
 
 void vm_merge(vm_t *target, vm_t *source) {
@@ -1090,7 +1051,7 @@ static link_t play_PGC_PG(vm_t *vm, int pgN) {
 }
 
 static link_t play_PGC_post(vm_t *vm) {
-  link_t link_values = { LinkNoLink, 0, 0, 0 };
+  link_t link_values;
 
 #ifdef TRACE
   fprintf(MSG_OUT, "libdvdnav: play_PGC_post:\n");
@@ -1227,6 +1188,15 @@ static link_t play_Cell_post(vm_t *vm) {
   if(cell->cell_cmd_nr != 0) {
     link_t link_values;
 
+/*  These asserts are now not needed.
+ *  Some DVDs have no cell commands listed in the PGC,
+ *  but the Cell itself points to a cell command that does not exist.
+ *  For this situation, just ignore the cell command and continue.
+ *
+ *  assert((vm->state).pgc->command_tbl != NULL);
+ *  assert((vm->state).pgc->command_tbl->nr_of_cell >= cell->cell_cmd_nr);
+ */
+
     if ((vm->state).pgc->command_tbl != NULL &&
         (vm->state).pgc->command_tbl->nr_of_cell >= cell->cell_cmd_nr) {
 #ifdef TRACE
@@ -1829,11 +1799,12 @@ static int set_PGN(vm_t *vm) {
   (vm->state).pgN = new_pgN;
 
   if((vm->state).domain == VTS_DOMAIN) {
+    playback_type_t *pb_ty;
     if((vm->state).TTN_REG > vm->vmgi->tt_srpt->nr_of_srpts)
       return 0; /* ?? */
-
-    vm_get_current_title_part(vm, &dummy, &part);
-    (vm->state).PTTN_REG = part;
+    pb_ty = &vm->vmgi->tt_srpt->title[(vm->state).TTN_REG - 1].pb_ty;
+      vm_get_current_title_part(vm, &dummy, &part);
+      (vm->state).PTTN_REG = part;
   }
   return 1;
 }
